<?php
/*
 * カレンダー取得
 *
 * 指定月の曜日、祝祭日のデータ(CalendarEntityの配列)を作成します。
 *
 * var 1.1 2008/04/19 第三条第二項　「国民の祝日」が日曜日に当たるときは、その日後においてその日に最も近い「国民の祝日」でない日を休日とする。に対応
 * ver 1.0 2007/11/19
 * t-fujishima@a-pock.co.jp
 */
define('HOLIDAY_FILE',"holiday.dat");

class CalendarDate {
	
	var $holiday;
	var $holiday_month; // 祝祭日のある月かどうか
	var $year; // 春分・秋分の日を何度も計算しないように
	var $vernal; // 春分
	var $autumnal; // 秋分
	
	function CalendarDate() {
		$this->__construct();
	}
	
	function __construct() {
		$this->holiday = array();
		$this->holiday_month = array_fill(1,12,false);
	}
	
	function init() {
		
		$ret = false;
		$this->year = null;
		$this->vernal = null;
		$this->autumnal = null;
		

		$file = fopen(HOLIDAY_FILE, 'r');

		if (!empty($file)) {

			while($read = fgetcsv($file, 1024, ',')) {
				$rec = array();
				
				$tmp = split('/', $read[0]);
				$rec['month'] = $tmp[0];
				$rec['day'] = $tmp[1];
				
				$rec['title'] = $read[1];
				array_push($this->holiday, $rec);
				
				$this->holiday_month[$tmp[0]] = true;
			}
			if (count($this->holiday)) {
				$ret = true;
			}
		}
		return $ret;
	}
	
	function getMonth($year, $month) {
		
		$ret = array();
		$week_count = 1; // その月の何週目
		$monday_count = 1; // その月の何回目の月曜日
		$tmp = date('j', mktime(0, 0, 0, $month  + 1, 0, $year)); // 末日を取得
		
		for ($i = 1; $i <= $tmp; $i++) {

			$ent = new CalendarEntity();
			$w = date('w', strtotime($year."/".$month."/".$i));
			$ent->setWeek($w);
			$ent->setWeekCount($week_count);

			// 祝祭日設定
			$r = $this->getHolidayTitle($year, $month, $i, $w, $monday_count);
			if (NULL !== $r) {
				$ent->setHoliday(true);
				$ent->setTitle($r);
			} else {
				$ent->setHoliday(false);
				$ent->setTitle('');
			}

			// 土日月設定
			if ($w == 0) {
				$ent->setSunday(true);
			} else {
				$ent->setSunday(false);
			}
			if ($w == 6) {
				$ent->setSaturday(true);
				$week_count++; // 土曜日なら翌週へ
			} else {
				$ent->setSaturday(false);
			}
			if ($w == 1) {
				$ent->setMondayCount($monday_count);
				$monday_count++;
			} else {
				$ent->setMondayCount(0);
			}

			array_push($ret, $ent);

		}

		return $ret;

	}

	/**
	 *
	 * 祝祭日かどうかチェックして祝祭日ならTitleを返す。
	 *
	 *
	 * @reference http://www.h3.dion.ne.jp/~sakatsu/holiday_topic.htm
	 * @reference http://www.excite.co.jp/News/bit/00091166184792.html
	 *
	 */
	function getHolidayTitle($year, $month, $day, $week, $mondaycount) {

		$ret = NULL;
		$count = count($this->holiday);
		
		for ($i = 0; $i < $count; $i++) {
			if ($this->holiday[$i]['month'] == $month) {
				// 祝日
				if ($this->holiday[$i]['day'] == $day) {
					$ret = $this->holiday[$i]['title'];
					break;
				// ハッピーマンデー
				} else if ( ($week == 1) && (0 === strpos($this->holiday[$i]['day'], 'M')) ) {
					$count = substr($this->holiday[$i]['day'], 1, 1);
					if ($count == $mondaycount) {
						$ret = $this->holiday[$i]['title'];
						break;
					}
				// 翌日が秋分の日で前日がハッピーマンデーなら
				} else if ( ($month == 9) && ($week == 2)
							 && (($day + 1) == $this->getAutumnalEquinoxDate($year))
							 && (0 === strpos($this->holiday[$i]['day'], 'M')) ) {
					$count = substr($this->holiday[$i]['day'], 1, 1);
					if ($count == $mondaycount-1) {
						$ret = "国民の祝日";
					}
				}
			}
		}

		// 春分・秋分の日
		if ($this->year != $year) {
			$this->year = $year;
			$this->autumnal = $this->getAutumnalEquinoxDate($year);
			$this->vernal = $this->getVernalEquinoxDate($year);
		}
		if ($month == 9) {
			if ($day == $this->autumnal ) {
				$ret = "秋分の日";
			}
		} else if ($month == 3) {
			if ($day == $this->vernal) {
				$ret = "春分の日";
			}
		}

		// 振替休日チェック
		if ( ($ret === NULL) && ($week == 1) ) { // 月曜日
			$tmp = NULL;
			if ($day > 1) {
				$tmp = $this->getHolidayTitle($year, $month, $day -1, 0, $mondaycount);
			} else if ($month > 1) {
				$j = date('j', mktime(0, 0, 0, $month, 0, $year)); // 末日を取得
				$tmp = $this->getHolidayTitle($year, $month - 1, $j, 0, $mondaycount);
			}
			if ( ($tmp !== NULL) && ($tmp != "国民の祝日") && ($tmp != "振替休日") ) {
				$ret = "振替休日";
			}
		} else if ( ($ret === NULL) && ($week == 2) ) { // 火曜日
			// 前日が休日、その前も休日 2008/5など
			$tmp1 = $this->getHolidayTitle($year, $month, $day -1, 0, $mondaycount);
			$tmp2 = $this->getHolidayTitle($year, $month, $day -2, 0, $mondaycount);
			if ( ($tmp1 !== NULL) && ($tmp2 !== NULL) ) {
				$ret = "振替休日";
			}
		} else if ( ($ret === NULL) && ($week == 3) ) { // 水曜日
			// 前日が休日、その前も休日、さらにその前も。 2009/5など
			$tmp1 = $this->getHolidayTitle($year, $month, $day -1, 0, $mondaycount);
			$tmp2 = $this->getHolidayTitle($year, $month, $day -2, 0, $mondaycount);
			$tmp3 = $this->getHolidayTitle($year, $month, $day -3, 0, $mondaycount);
			if ( ($tmp1 !== NULL) && ($tmp2 !== NULL) && ($tmp3 !== NULL) ) {
				$ret = "振替休日";
			}
		}
		return $ret;
	}
	
	function getVernalEquinoxDate($year) {
		$ret = (int)(20.8357 + 0.242194 * ($year - 1980) - (int)(($year - 1980) / 4));
		return $ret;
	}

	function getAutumnalEquinoxDate($year) {
		$ret = (int)(23.2488 + 0.242194 * ($year - 1980) - (int)(($year - 1980) / 4));
		return $ret;
	}


}

/*
 * カレンダーの計算結果を１日ごとに格納
 *
 */
class CalendarEntity {
	var $week; // 曜日
	var $weekcount; // その月の何週か
	var $mondaycount; // その月の何回目の月曜日(月曜日以外は0)
	var $holiday; // 祝祭日
	var $sunday;
	var $saturday;
	var $title; // 祝祭日なら何の日か。
	var $contents;
	
	function getWeek() {
		return $this->week;
	}
	function isHoliday() {
		return $this->holiday;
	}
	function isSunday() {
		return $this->sunday;
	}
	function isSaturday() {
		return $this->saturday;
	}
	function getTitle() {
		return $this->title;
	}
	function getContents() {
		return $this->contents;
	}
	function getWeekCount() {
		return $this->weekcount;
	}
	function getMondayCount() {
		return $this->mondaycount;
	}
	
	function setWeek($in) {
		$this->week = $in;
	}
	function setHoliday($in) {
		$this->holiday = $in;
	}
	function setSunday($in) {
		$this->sunday = $in;
	}
	function setSaturday($in) {
		$this->saturday = $in;
	}
	function setTitle($in) {
		$this->title = $in;
	}
	function setContents($in) {
		$this->contents = $in;
	}
	function setWeekCount($in) {
		$this->weekcount = $in;
	}
	function setMondayCount($in) {
		$this->mondaycount = $in;
	}

}

?>